package com.atm.bondowowo;

import android.app.Activity
import android.app.Application
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.widget.Toast
import androidx.annotation.RequiresApi
import com.atm.bondowowo.ui.login.LoginActivity
import com.atm.bondowowo.utils.BackgroundJWTChecker
import com.atm.bondowowo.utils.RandomInputBroadcastReceiver
import com.atm.bondowowo.utils.UserPreferencesUtil
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.SupervisorJob

class BondowowoApp : Application() {

    private val applicationScope = CoroutineScope(SupervisorJob())
    private var jwtChecker: BackgroundJWTChecker? = null

    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    override fun onCreate() {
        super.onCreate()
        val myApplication = applicationContext as BondowowoApp
        val appScope = myApplication.getApplicationScope()
        initBackgroundJWTChecker(this, appScope, myApplication)
    }


    private fun initBackgroundJWTChecker(context: Context, scope: CoroutineScope, app: BondowowoApp) {
        jwtChecker = BackgroundJWTChecker()
        jwtChecker?.startChecking(scope, context, object : BackgroundJWTChecker.Callback {
            override fun onTokenVerified() {
                // Continu app
            }

            override fun onTokenInvalid() {
//                Toast.makeText(context, "Your session has ended", Toast.LENGTH_SHORT).show()
                UserPreferencesUtil.removeJWT(context)
                UserPreferencesUtil.removeEmail(context)
                val intent = Intent(context, LoginActivity::class.java)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
                context.startActivity(intent)
                (context as? Activity)?.finish()
            }

            override fun onNetworkError() {
//                Toast.makeText(context, "Re-login to use full feature", Toast.LENGTH_SHORT).show()
            }
        })
    }

    private fun getApplicationScope(): CoroutineScope {
        return applicationScope
    }

}
