package com.atm.bondowowo

import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.os.Bundle
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AppCompatActivity
import androidx.navigation.findNavController
import androidx.navigation.fragment.NavHostFragment
import androidx.navigation.ui.setupWithNavController
import com.atm.bondowowo.databinding.ActivityMainBinding
import com.atm.bondowowo.ui.login.LoginActivity
import com.atm.bondowowo.utils.DialogHelper
import com.atm.bondowowo.utils.NetworkStateLiveData
import com.atm.bondowowo.utils.NetworkUtils
import com.atm.bondowowo.utils.RandomInputBroadcastReceiver
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

class MainActivity : AppCompatActivity() {

    private lateinit var binding: ActivityMainBinding
    lateinit var navHostFragment: NavHostFragment
//    private val randomInputReceiver = RandomInputBroadcastReceiver(this)

    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val networkStateLiveData = NetworkStateLiveData(this)
        networkStateLiveData.observe(this) { isConnected ->
            if (!isConnected) {
                DialogHelper.showNoInternetDialog(this)
            }
        }

        CoroutineScope(Dispatchers.Main).launch {
            if (!isUserAuthenticated()) {
                navigateToLoginActivity()
            } else {
                navHostFragment =
                    supportFragmentManager.findFragmentById(R.id.nav_host_fragment) as NavHostFragment
                val navController = navHostFragment.navController
                binding.bottomNavLayout.bottomNavigation.setupWithNavController(navController)
            }
        }
    }

    override fun onSupportNavigateUp(): Boolean {
        val navController = findNavController(R.id.nav_host_fragment)
        return navController.navigateUp() || super.onSupportNavigateUp()
    }

    private suspend fun isUserAuthenticated(): Boolean {
        val sharedPreferences = getSharedPreferences("AUTH_PREFS", MODE_PRIVATE)
        val token = sharedPreferences.getString("JWT_TOKEN", null)

        return token != null && isTokenValid(token)
    }

    private fun navigateToLoginActivity() {
        startActivity(Intent(this, LoginActivity::class.java))
        finish()
    }

    private suspend fun isTokenValid(token: String): Boolean {
        return try {
            val response = NetworkUtils.apiService.verifyToken("Bearer $token")
            if (response.isSuccessful) {
                val responseBody = response.body()
                val isString = responseBody is String
                return !isString
            } else {
                Toast.makeText(
                    this@MainActivity,
                    "Session Expired, Please Re login",
                    Toast.LENGTH_SHORT
                ).show()
                false
            }
        } catch (e: Exception) {
            Toast.makeText(
                this@MainActivity,
                "An error occurred, Please Re login",
                Toast.LENGTH_SHORT
            ).show()
            false
        }
    }

}
