package com.atm.bondowowo.data.repository

import com.atm.bondowowo.data.local.dao.TransactionDao
import com.atm.bondowowo.data.local.database.AppDatabase
import com.atm.bondowowo.data.local.entity.TransactionEntity
import com.atm.bondowowo.data.model.Transaction

class TransactionRepository(private val database: AppDatabase) {

    fun getAllTransactions(): List<Transaction> {
        return database.transactionDao().getAll().map { it.toTransaction() }
    }

    fun insertTransaction(transaction: Transaction) {
        database.transactionDao().insertAll(transaction.toTransactionEntity())
    }

    fun insertTransactions(transactions: List<Transaction>) {
        database.transactionDao().insertAll(*transactions.map { it.toTransactionEntity() }.toTypedArray())
    }

    fun deleteTransaction(transaction: Transaction) {
        database.transactionDao().delete(transaction.toTransactionEntity())
    }

    fun getTransaction(id: Int): Transaction {
        return database.transactionDao().get(id).toTransaction()
    }

    fun updateTransaction(transaction: Transaction) {
        database.transactionDao().update(transaction.toTransactionEntity())
    }

    private fun TransactionEntity.toTransaction(): Transaction {
        return Transaction(
            id = this.id,
            date = this.date,
            nama = this.nama,
            kategori = this.kategori,
            nominal = this.nominal,
            lokasi = this.lokasi
        )
    }

    private fun Transaction.toTransactionEntity(): TransactionEntity {
        return TransactionEntity(
            id = this.id,
            date = this.date,
            nama = this.nama,
            kategori = this.kategori,
            nominal = this.nominal,
            lokasi = this.lokasi
        )
    }
}