package com.atm.bondowowo.ui.graph

import android.content.Context
import android.graphics.Color
import android.view.View
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.lifecycle.LiveData
import com.atm.bondowowo.R
import com.atm.bondowowo.data.model.Transaction
import com.github.mikephil.charting.charts.PieChart
import androidx.lifecycle.Observer
import com.github.mikephil.charting.animation.Easing
import com.github.mikephil.charting.components.Legend
import com.github.mikephil.charting.data.PieData
import com.github.mikephil.charting.data.PieDataSet
import com.github.mikephil.charting.data.PieEntry
import com.github.mikephil.charting.formatter.PercentFormatter

class ChartView(context: Context) : View(context) {

    lateinit var pieChart: PieChart

    override fun onFinishInflate() {
        super.onFinishInflate()
        pieChart = findViewById(R.id.pieChart)

    }

    private val observer = Observer<List<Transaction>> { newData ->
        newData.let {
            updateChart(it)
        }
    }

    fun setChartData(data: LiveData<List<Transaction>>) {
        data.observeForever(observer)
    }

    private fun updateChart(transactions: List<Transaction>) {
        val groupedTransactions = transactions.groupBy { it.kategori }
        val entries = groupedTransactions.map { (group, transactions) ->
            val totalQuantity = transactions.sumOf { it.nominal?.toInt() ?: 0 }
            PieEntry(totalQuantity.toFloat(), group)
        }


        if (entries.isNotEmpty()) {
//            pieChart.visibility = VISIBLE
            pieChart.description.text = "Amount each Category"
            pieChart.description.textSize = 20f
            pieChart.description.textColor = Color.BLACK
            pieChart.holeRadius = 40f

            val pieDataSet = PieDataSet(entries, "")
            pieDataSet.colors =
                listOf(ContextCompat.getColor(context, R.color.primary1), ContextCompat.getColor(context, R.color.primary3))
            pieDataSet.valueTextSize = 24f
            pieDataSet.valueTextColor = Color.WHITE
            pieDataSet.sliceSpace = 3f
            pieDataSet.selectionShift = 15f

            pieChart.legend.isEnabled = true
            pieChart.legend.orientation = Legend.LegendOrientation.HORIZONTAL
            pieChart.legend.horizontalAlignment = Legend.LegendHorizontalAlignment.CENTER
            pieChart.legend.verticalAlignment = Legend.LegendVerticalAlignment.TOP
            pieChart.legend.formSize = 24f
            pieChart.legend.textSize = 16f
            pieChart.animateY(1500, Easing.EaseInOutQuad)
            pieChart.animateX(1500, Easing.EaseInOutQuad)
            pieChart.isHighlightPerTapEnabled = true
            pieChart.isRotationEnabled = true
            pieChart.setEntryLabelColor(Color.WHITE)
            pieChart.setEntryLabelTextSize(24f)

            val pieData = PieData(pieDataSet)
            pieChart.data = pieData
            pieChart.invalidate()
        } else {
//            pieChart.visibility = GONE
        }
    }

}
