package com.atm.bondowowo.ui.graph

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.atm.bondowowo.data.repository.TransactionRepository
import com.atm.bondowowo.data.model.Transaction
import kotlinx.coroutines.launch

class GraphViewModel(
    private val transactionRepository: TransactionRepository
) : ViewModel() {


    private val _transactionData = MutableLiveData<List<Transaction>>()
    val transactionData: LiveData<List<Transaction>> = _transactionData
    private val toastMessage = MutableLiveData<String>()

    init {
        loadTransactionData()
    }

    private fun loadTransactionData() {
        viewModelScope.launch {
            try {
                val transactions = transactionRepository.getAllTransactions()
                _transactionData.postValue(transactions)
            } catch (e: Exception) {
                showToast(e.message ?: "Failed to load transaction data")
            } finally {
            }
        }
    }

    fun showToast(message: String) {
        toastMessage.value = message
    }

}