package com.atm.bondowowo.ui.scan

import android.Manifest
import android.app.Activity
import android.app.AlertDialog
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.provider.MediaStore
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.TextView
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProvider
import com.atm.bondowowo.R
import com.atm.bondowowo.data.local.database.AppDatabase
import com.atm.bondowowo.data.repository.TransactionRepository
import com.atm.bondowowo.ui.twibbon.TwibbonActivity
import com.atm.bondowowo.utils.UserPreferencesUtil
import com.budiyev.android.codescanner.CodeScanner
import com.budiyev.android.codescanner.CodeScannerView
import com.budiyev.android.codescanner.DecodeCallback
import com.google.zxing.BinaryBitmap
import com.google.zxing.LuminanceSource
import com.google.zxing.MultiFormatReader
import com.google.zxing.RGBLuminanceSource
import com.google.zxing.Reader
import com.google.zxing.common.HybridBinarizer
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import java.io.BufferedInputStream
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import java.io.IOException
import java.io.InputStream

class ScanFragment : Fragment() {
    private lateinit var scanViewModel: ScanViewModel

    private lateinit var tvResult: TextView
    private var WRITE_EXTERNAL_STORAGE_PERMISSION_CODE: Int = 1
    private var READ_EXTERNAL_STORAGE_PERMISSION_CODE: Int = 2

    private val CAMERA_PERMISSION_REQUEST_CODE = 100
    private lateinit var codeScanner: CodeScanner
    private lateinit var scannedFile: File

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_scan, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        val database = AppDatabase.getInstance(requireContext())
        val transactionRepository = TransactionRepository(database)
        val scanViewModelFactory = ScanViewModelFactory(transactionRepository)
        scanViewModel = ViewModelProvider(this, scanViewModelFactory)[ScanViewModel::class.java]
        val scannerView = view.findViewById<CodeScannerView>(R.id.scanner)
        tvResult = view.findViewById(R.id.tv_result) //

        val btnUpload = view.findViewById<Button>(R.id.btn_upload)
        btnUpload.setOnClickListener {
            openGallery()
        }

        val btnTwibbon = view.findViewById<Button>(R.id.btn_filter)
        btnTwibbon.setOnClickListener {
            val intent = Intent(requireContext(), TwibbonActivity::class.java)
            startActivity(intent)
        }

        // Check camera permission
        if (ContextCompat.checkSelfPermission(
                requireContext(),
                Manifest.permission.CAMERA
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(
                requireActivity(),
                arrayOf(Manifest.permission.CAMERA),
                CAMERA_PERMISSION_REQUEST_CODE
            )
        } else {
            initializeCodeScanner(scannerView)
        }

        // for file
        if (ContextCompat.checkSelfPermission(
                requireContext(),
                Manifest.permission.WRITE_EXTERNAL_STORAGE
            ) == PackageManager.PERMISSION_DENIED
        ) {
            ActivityCompat.requestPermissions(
                requireActivity(),
                arrayOf(Manifest.permission.WRITE_EXTERNAL_STORAGE),
                WRITE_EXTERNAL_STORAGE_PERMISSION_CODE
            )
        } else if (ContextCompat.checkSelfPermission(
                requireContext(),
                Manifest.permission.READ_EXTERNAL_STORAGE
            ) == PackageManager.PERMISSION_DENIED
        ) {
            ActivityCompat.requestPermissions(
                requireActivity(),
                arrayOf(Manifest.permission.READ_EXTERNAL_STORAGE),
                READ_EXTERNAL_STORAGE_PERMISSION_CODE
            )
        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == CAMERA_PERMISSION_REQUEST_CODE) {
            if (grantResults.isNotEmpty() && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                val scannerView = requireView().findViewById<CodeScannerView>(R.id.scanner)
                initializeCodeScanner(scannerView)
            } else {
                Toast.makeText(requireContext(), "Camera permission denied", Toast.LENGTH_SHORT)
                    .show()
            }
        } else if (requestCode == WRITE_EXTERNAL_STORAGE_PERMISSION_CODE) {
            if (grantResults.isNotEmpty() && grantResults[0] == PackageManager.PERMISSION_DENIED) {
                Toast.makeText(
                    requireContext(),
                    "Anda perlu memberikan semua izin untuk menggunakan aplikasi ini.",
                    Toast.LENGTH_SHORT
                ).show()
                requireActivity().finish()
            }
        } else if (requestCode == READ_EXTERNAL_STORAGE_PERMISSION_CODE) {
            if (grantResults.isNotEmpty() && grantResults[0] == PackageManager.PERMISSION_DENIED) {
                Toast.makeText(
                    requireContext(),
                    "Anda perlu memberikan semua izin untuk menggunakan aplikasi ini.",
                    Toast.LENGTH_SHORT
                ).show()
                requireActivity().finish()
            }
        }
    }

    private fun initializeCodeScanner(scannerView: CodeScannerView) {
        codeScanner = CodeScanner(requireActivity(), scannerView)
        codeScanner.startPreview()

        codeScanner.decodeCallback = DecodeCallback {
            requireActivity().runOnUiThread {
                val contents = it.text
                tvResult.text = contents
                val convertedFile = saveByteArrayToFile(it.rawBytes)
                if (convertedFile !== null) {
                    scannedFile = convertedFile
                    Toast.makeText(requireContext(), "Scanned: ${it.text}", Toast.LENGTH_LONG)
                        .show()
                    val scanOptions = arrayOf<String>("Ya", "Tidak")
                    AlertDialog.Builder(requireContext())
                        .setTitle("Scan Berhasil!\nApakah ingin menyimpan\n proses scan?")
                        .setItems(scanOptions) { _, which ->
                            when (which) {
                                0 -> setToTransaction()
                                1 -> codeScanner.startPreview()
                            }
                        }
                        .create().show()
                } else {
                    Toast.makeText(
                        requireContext(),
                        "Failed to capture Image, please retry",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }
        }
        scannerView.setOnClickListener {
            codeScanner.startPreview()
        }
    }

    private fun setToTransaction() {
        val tokenJWT = UserPreferencesUtil.getJWT(requireContext())
        val requestFile = RequestBody.create(MediaType.parse("image/*"), scannedFile)
        val imagePart = MultipartBody.Part.createFormData("file", scannedFile.name, requestFile)
        if (tokenJWT != null) {
            scanViewModel.uploadScan(tokenJWT, imagePart, requireContext(), scannedFile)
        } else {
            Toast.makeText(
                requireContext(),
                "Unable to get auth, please Log In again",
                Toast.LENGTH_SHORT
            ).show()
        }

    }

    private fun openGallery() {
        Log.d("ScanFragment", "Fungsi openGallery() dipanggil")
        val galleryIntent =
            Intent(Intent.ACTION_PICK, MediaStore.Images.Media.EXTERNAL_CONTENT_URI)
        resultLauncherGallery.launch(galleryIntent)
    }

    private val resultLauncherGallery =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            if (result.resultCode == Activity.RESULT_OK) {
                val data: Intent? = result.data

                val imageUri = data!!.data!!
                val imagePath = convertMediaUriToPath(imageUri)
                Log.d("ScanFragment", "Berhasil convertMediaUriToPath")
                val imgFile = File(imagePath)
                scannedFile = imgFile
                scanImageQRCode(imgFile)
                Log.d("ScanFragment", "Berhasil scanImageQRCode")
            } else {
                Toast.makeText(requireContext(), "Result Not Found", Toast.LENGTH_LONG).show()
            }
        }

    private fun convertMediaUriToPath(uri: Uri): String {
        val proj = arrayOf(MediaStore.Images.Media.DATA)
        val cursor = requireActivity().contentResolver.query(uri, proj, null, null, null)
        val columnIndex = cursor!!.getColumnIndexOrThrow(MediaStore.Images.Media.DATA)
        cursor.moveToFirst()
        val path = cursor.getString(columnIndex)
        cursor.close()
        return path
    }

    private fun scanImageQRCode(file: File) {
        scannedFile = file
        val inputStream: InputStream = BufferedInputStream(FileInputStream(file))
        val bitmap = BitmapFactory.decodeStream(inputStream)
        val decoded = scanQRImage(bitmap)
        Log.i("QrTest", "Decoded string=$decoded")
    }

    private fun scanQRImage(bMap: Bitmap): String? {
        var contents: String? = null
        val intArray = IntArray(bMap.width * bMap.height)
        //copy pixel data from the Bitmap into the 'intArray' array
        bMap.getPixels(intArray, 0, bMap.width, 0, 0, bMap.width, bMap.height)
        val source: LuminanceSource = RGBLuminanceSource(bMap.width, bMap.height, intArray)
        val bitmap = BinaryBitmap(HybridBinarizer(source))
        val reader: Reader = MultiFormatReader()
        try {
            val result: com.google.zxing.Result = reader.decode(bitmap)
            Log.d("scanQRImage", "berhasil yang bagian Result")
            contents = result.text
            tvResult.text = contents

            val scanOptions = arrayOf<String>("Ya", "Tidak")
            AlertDialog.Builder(requireContext())
                .setTitle("Scan Berhasil!\nApakah ingin mengulagi proses Scan?")
                .setItems(scanOptions) { _, which ->
                    when (which) {
                        0 -> codeScanner.startPreview()
                        1 -> setToTransaction()
                    }
                }
                .create().show()

        } catch (e: Exception) {
            Log.e("QrTest", "Error decoding qr code", e)
            Toast.makeText(
                requireContext(),
                "Error decoding QR Code, Mohon pilih gambar QR Code yang benar!",
                Toast.LENGTH_SHORT
            ).show()
        }
        return contents
    }

    fun saveByteArrayToFile(byteArray: ByteArray): File? {
        val directory =
            Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES)
        val fileName = "temp_image_${System.currentTimeMillis()}.png" // Unique filename
        val file = File(directory, fileName)

        try {
            val outputStream = FileOutputStream(file)
            outputStream.write(byteArray)
            outputStream.flush()
            outputStream.close()
            return file
        } catch (e: IOException) {
            e.printStackTrace()
        }
        return null
    }

}
