package com.atm.bondowowo.ui.scan

import android.content.Context
import android.widget.Toast
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.atm.bondowowo.data.model.ScanItemData
import com.atm.bondowowo.data.model.Transaction
import com.atm.bondowowo.data.repository.TransactionRepository
import com.atm.bondowowo.utils.NetworkUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import okhttp3.MultipartBody
import java.io.File
import java.time.LocalDateTime

class ScanViewModel(
    private val transactionRepository: TransactionRepository
) : ViewModel() {
    fun uploadScan(tokenJWT: String, imagePart: MultipartBody.Part, context: Context, file: File) {
        viewModelScope.launch {
            try {
                val response = withContext(Dispatchers.IO) {
                    tokenJWT.let {
                        NetworkUtils.apiService.uploadScan(
                            "Bearer $it",
                            imagePart
                        )
                    }
                }
                if (response.isSuccessful) {
                    val scanResponse = response.body()?.items?.items
                    val scanTransaction = scanResponse?.let { convertFromScanResponse(it) }
                    if (scanTransaction != null) {
                        transactionRepository.insertTransaction(scanTransaction)
                        Toast.makeText(
                            context,
                            "Successfully new transaction",
                            Toast.LENGTH_SHORT
                        ).show()
                    } else {
                        Toast.makeText(
                            context,
                            "Unable to add data to phone storage",
                            Toast.LENGTH_SHORT
                        ).show()
                    }
                } else {
                    Toast.makeText(
                        context,
                        "Failed, please re-Login",
                        Toast.LENGTH_SHORT
                    ).show()
                }
                file.delete()
            } catch (e: Exception) {
                e.printStackTrace()
                Toast.makeText(context, "Unable to connect server...", Toast.LENGTH_SHORT)
                    .show()
            }

        }
    }

    private fun convertFromScanResponse(itemList: List<ScanItemData>): Transaction {
        val name = "Transaksi - " + itemList.map { it.name }.joinToString(", ")
        val kategori = if (itemList.size % 2 == 0) {
            "pemasukan"
        } else {
            "pengeluaran"
        }
        val nominal = itemList.fold(0.0) { acc, item -> acc + (item.qty * item.price) }
        return Transaction(
            null,
            nama = name,
            date = LocalDateTime.now(),
            kategori = kategori,
            nominal = nominal,
            lokasi = "Toko ${itemList[0].name}"
        )
    }
}