package com.atm.bondowowo.ui.settings

import android.app.AlertDialog
import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.LinearLayout
import androidx.fragment.app.Fragment
import com.atm.bondowowo.ui.login.LoginActivity
import com.atm.bondowowo.R
import com.atm.bondowowo.data.local.database.AppDatabase
import com.atm.bondowowo.utils.ExportToFileUtil
import com.atm.bondowowo.utils.SendEmailUtil
import com.atm.bondowowo.utils.UserPreferencesUtil


class SettingsFragment : Fragment() {
    private lateinit var linearLayout: LinearLayout
    private lateinit var btn_export: Button
    private lateinit var btn_send_email: Button
    private lateinit var btn_logout: Button
    private lateinit var btn_random: Button
    private lateinit var database: AppDatabase

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        val view = inflater.inflate(R.layout.fragment_settings, container, false)
        linearLayout = view.findViewById(R.id.layout_settings)
        btn_export = view.findViewById(R.id.button_export_to_file)
        btn_send_email = view.findViewById(R.id.button_send_via_email)
        btn_logout = view.findViewById(R.id.button_logout_settings)
        btn_random = view.findViewById(R.id.button_randomize_input)

        val context = requireActivity().applicationContext
        database = AppDatabase.getInstance(context)

        btn_export.setOnClickListener {
            showExportFileDialog()
        }

        btn_send_email.setOnClickListener {
            SendEmailUtil.sendEmailWithAttachment(requireContext())
        }

        btn_logout.setOnClickListener {
            UserPreferencesUtil.removeJWT(requireContext())
            UserPreferencesUtil.removeEmail(requireContext())
            val intent =
                Intent(requireContext(), LoginActivity::class.java)
            startActivity(intent)
        }

        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        btn_random.setOnClickListener {
            sendRandomInputBroadcast()
        }
    }

    private fun sendRandomInputBroadcast() {
        val randomInput = (0..100).random().toString()
        val intent = Intent("com.atm.bondowowo.RANDOM_INPUT_TRANSACTION")
        intent.putExtra("input_value", randomInput)
        context?.sendBroadcast(intent)
    }

    private fun showExportFileDialog() {
        val options = arrayOf("XLSX", "XLS")
        val builder = AlertDialog.Builder(requireContext())
        builder.setTitle("Choose file format")
        builder.setItems(options, DialogInterface.OnClickListener { dialog, which ->
            when (which) {
                0 -> exportToFormat("xlsx")
                1 -> exportToFormat("xls")
            }
            dialog.dismiss()
        })
        builder.show()
    }

    private fun exportToFormat(fileFormat: String) {
        val listOfTransactions = database.transactionDao().getAll()
        ExportToFileUtil.exportDataToFile(requireContext(), listOfTransactions, fileFormat)
    }


}