package com.atm.bondowowo.ui.transaction

import android.content.Intent
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.Observer
import com.atm.bondowowo.MainActivity
import com.atm.bondowowo.R
import com.atm.bondowowo.utils.DialogHelper
import com.atm.bondowowo.utils.NetworkHelper
import com.atm.bondowowo.utils.NetworkStateLiveData

class TransactionActivity : AppCompatActivity(), TransactionInputFragment.TransactionInputListener {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_transaction)

        val networkStateLiveData = NetworkStateLiveData(this)
        networkStateLiveData.observe(this, Observer { isConnected ->
            if (!isConnected) {
                DialogHelper.showNoInternetDialog(this)
            }
        })

        val transactionInputFragment = TransactionInputFragment()

        // Get the id passed from the intent
        val id = intent.getIntExtra("id", -1)

        // Get random number if exist
        val randomNumber = intent.getStringExtra("randomNumber")

        if (id != -1) {
            // Pass the id to the fragment if id found from intent
            val args = Bundle().apply {
                putInt("id", id)
            }
            transactionInputFragment.arguments = args
        }

        if (randomNumber != null && id == -1) {
            val args = Bundle().apply {
                putString("randomNumber", randomNumber)
            }
            transactionInputFragment.arguments = args
        }

        transactionInputFragment.setListener(this)

        supportFragmentManager.beginTransaction()
            .add(R.id.fragment_container, transactionInputFragment)
            .commit()
    }

    override fun onTransactionSubmitted() {
        startActivity(Intent(this, MainActivity::class.java))
        finish()
    }


}
