package com.atm.bondowowo.ui.transaction

import android.app.AlertDialog
import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.core.content.ContextCompat.registerReceiver
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProvider
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.atm.bondowowo.R
import com.atm.bondowowo.adapter.TransactionAdapter
import com.atm.bondowowo.data.local.database.AppDatabase
import com.atm.bondowowo.data.local.entity.TransactionEntity
import com.atm.bondowowo.data.repository.TransactionRepository
import com.atm.bondowowo.ui.graph.GraphViewModel
import com.atm.bondowowo.ui.graph.GraphViewModelFactory
import com.atm.bondowowo.utils.GoogleMapsUtil
import com.atm.bondowowo.utils.RandomInputBroadcastReceiver
import com.google.android.material.floatingactionbutton.FloatingActionButton


class TransactionFragment : Fragment() {

    private lateinit var recyclerView: RecyclerView
    private lateinit var fab: FloatingActionButton
    private var list = mutableListOf<TransactionEntity>()
    private lateinit var adapter: TransactionAdapter
    private lateinit var database: AppDatabase
    private lateinit var transactionViewModel: TransactionViewModel

    private lateinit var randomInputReceiver: RandomInputBroadcastReceiver


    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        val view = inflater.inflate(R.layout.fragment_transaction, container, false)
        recyclerView = view.findViewById(R.id.cart_list)
        fab = view.findViewById(R.id.fab)

        database = AppDatabase.getInstance(requireContext())
        val transactionRepository = TransactionRepository(database)
        val transactionViewModelFactory = TransactionViewModelFactory(transactionRepository)
        transactionViewModel =
            ViewModelProvider(this, transactionViewModelFactory)[TransactionViewModel::class.java]

        val context = requireActivity().applicationContext
        adapter = TransactionAdapter(list)

        adapter.setDialog(object : TransactionAdapter.Dialog {
            override fun onClick(position: Int) {
                val dialog = AlertDialog.Builder(requireContext())
                dialog.setTitle(list[position].nama)
                dialog.setItems(
                    R.array.transaction_option
                ) { dialog, which ->
                    if (which == 0) {
                        val intent =
                            Intent(requireContext(), TransactionActivity::class.java)
                        intent.putExtra("id", list[position].id)
                        startActivity(intent)
                    } else if (which == 1) {
                        showDeleteConfirmationDialog(position)
                    } else if (which == 2) {
                        list[position].id?.let {
                            GoogleMapsUtil.openGoogleMaps(
                                requireContext(),
                                it
                            )
                        }
                    } else {
                        dialog.dismiss()
                    }
                }
                val dialogView = dialog.create()
                dialogView.show()
            }
        })

        recyclerView.adapter = adapter
        recyclerView.layoutManager = LinearLayoutManager(context, RecyclerView.VERTICAL, false)
        recyclerView.addItemDecoration(
            DividerItemDecoration(context, RecyclerView.VERTICAL)
        )

        fab.setOnClickListener {
            val intent =
                Intent(requireContext(), TransactionActivity::class.java)
            if (transactionViewModel.randomNumber != "") {
                intent.putExtra("randomNumber", transactionViewModel.randomNumber)
                transactionViewModel.randomNumber = ""
            }
            startActivity(intent)
        }

        return view
    }

    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        randomInputReceiver = RandomInputBroadcastReceiver(requireContext(), this)
        registerRandomInputReceiver()
    }

    override fun onResume() {
        super.onResume()
        getData()
    }

    private fun getData() {
        list.clear()
        list.addAll(database.transactionDao().getAll())
        adapter.notifyDataSetChanged()
    }

    fun setRandomNumber(num: String) {
        transactionViewModel.randomNumber = num
    }

    private fun showDeleteConfirmationDialog(position: Int) {
        val builder = AlertDialog.Builder(requireContext())
        builder.setTitle("Delete Transaction")
            .setMessage("Apakah anda ingin menghapus transaksi ini?")
            .setPositiveButton("Yes") { dialog, which ->
                deleteTransaction(position)
            }
            .setNegativeButton("No") { dialog, which ->
                dialog.dismiss()
            }
            .create()
            .show()
    }

    private fun deleteTransaction(position: Int) {
        database.transactionDao().delete(list[position])
        getData()
    }

    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    private fun registerRandomInputReceiver() {
        registerReceiver(
            requireContext(),
            randomInputReceiver,
            IntentFilter("com.atm.bondowowo.RANDOM_INPUT_TRANSACTION"),
            ContextCompat.RECEIVER_EXPORTED
        )
    }

//    private fun unregisterRandomInputReceiver() {
//        unregisterReceiver(randomInputReceiver)
//    }
}
