package com.atm.bondowowo.ui.transaction

import android.Manifest
import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.location.Geocoder
import android.location.Location
import android.location.LocationListener
import android.location.LocationManager
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.Button
import android.widget.EditText
import android.widget.Spinner
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.core.app.ActivityCompat
import androidx.fragment.app.Fragment
import com.atm.bondowowo.R
import com.atm.bondowowo.data.local.database.AppDatabase
import com.atm.bondowowo.data.local.entity.TransactionEntity
import com.atm.bondowowo.utils.RandomInputBroadcastReceiver
import java.time.LocalDateTime

class TransactionInputFragment : Fragment() {
    private lateinit var nama_transaksi: EditText
    private lateinit var nominal_transaksi: EditText
    private lateinit var kategori_transaksi: Spinner

    private lateinit var lokasi_transaksi: EditText
    private lateinit var btnSave: Button
    private lateinit var database: AppDatabase
    private lateinit var listener: TransactionInputListener

    interface TransactionInputListener {
        fun onTransactionSubmitted()
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        val view = inflater.inflate(R.layout.fragment_transaction_input, container, false)

        nama_transaksi = view.findViewById(R.id.nama_transaksi)
        nominal_transaksi = view.findViewById(R.id.nomimal_transaksi)
        kategori_transaksi = view.findViewById(R.id.kategori_transaksi)
        lokasi_transaksi = view.findViewById(R.id.lokasi_transaksi)
        btnSave = view.findViewById(R.id.btn_save)

        database = AppDatabase.getInstance(requireContext())

        val categories = arrayOf("pemasukan", "pengeluaran")
        val adapter =
            ArrayAdapter(requireContext(), android.R.layout.simple_spinner_item, categories)
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        kategori_transaksi.adapter = adapter


        val args = arguments
        if (args != null && args.getInt("id", -1) != -1) {
            val id = args.getInt("id", -1)
            val transaction = database.transactionDao().get(id)

            kategori_transaksi.isEnabled = false
            kategori_transaksi.isClickable = false

            nama_transaksi.setText(transaction.nama)
            nominal_transaksi.setText(transaction.nominal.toString())
            kategori_transaksi.setSelection(categories.indexOf(transaction.kategori))
            lokasi_transaksi.setText(transaction.lokasi)
        } else if (args?.getString("randomNumber") != null) {
            setNominal(args.getString("randomNumber")!!)
        }

        btnSave.setOnClickListener {
            if (nama_transaksi.text.isNotEmpty() && nominal_transaksi.text.isNotEmpty() && lokasi_transaksi.text.isNotEmpty()) {
                val category = kategori_transaksi.selectedItem.toString()
                val transactionEntity =
                    if (args?.getInt("id", -1) != -1 && args?.getInt("id", -1) != null) {
                        TransactionEntity(
                            args?.getInt("id", 0),
                            LocalDateTime.now(),
                            nama_transaksi.text.toString(),
                            category,
                            nominal_transaksi.text.toString().toDouble(),
                            lokasi_transaksi.text.toString()
                        )
                    } else {
                        TransactionEntity(
                            null,
                            LocalDateTime.now(),
                            nama_transaksi.text.toString(),
                            category,
                            nominal_transaksi.text.toString().toDouble(),
                            lokasi_transaksi.text.toString()
                        )
                    }

                val id = args?.getInt("id", -1)
                if (id != -1 && id != null) {
                    database.transactionDao().update(transactionEntity)
                } else {
                    database.transactionDao().insertAll(transactionEntity)
                }

                listener.onTransactionSubmitted()
            } else {
                Toast.makeText(requireContext(), "Input invalid", Toast.LENGTH_SHORT).show()
            }
        }

        return view
    }

    @SuppressLint("MissingPermission")
    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        if (checkLocationPermission()) {
            locationManager =
                requireActivity().getSystemService(Context.LOCATION_SERVICE) as LocationManager
            locationManager.requestLocationUpdates(
                LocationManager.GPS_PROVIDER,
                0,
                0f,
                locationListener
            )
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
//        requireContext().unregisterReceiver(inputValueReceiver)
        if (this::locationManager.isInitialized) {
            locationManager.removeUpdates(locationListener)
        }
    }

    fun setListener(listener: TransactionInputListener) {
        this.listener = listener
    }


    fun setNominal(nominal: String) {
        nominal_transaksi.setText(nominal)
    }

    private lateinit var locationManager: LocationManager

    private val locationListener: LocationListener = object : LocationListener {
        override fun onLocationChanged(location: Location) {
            val geocoder = Geocoder(requireContext())
            val addresses = geocoder.getFromLocation(location.latitude, location.longitude, 1)

            if (addresses != null) {
                if (addresses.isNotEmpty()) {
                    val address = addresses[0].getAddressLine(0)
                    lokasi_transaksi.setText(address)
                }
            }
        }

        @Deprecated("Deprecated in Java")
        override fun onStatusChanged(provider: String?, status: Int, extras: Bundle?) {
        }

        override fun onProviderEnabled(provider: String) {}

        override fun onProviderDisabled(provider: String) {}
    }

    private fun checkLocationPermission(): Boolean {
        if (ActivityCompat.checkSelfPermission(
                requireContext(),
                Manifest.permission.ACCESS_FINE_LOCATION
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            requestPermissions(arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), 1)
            return false
        }
        return true
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == 1) {
            if (grantResults.isNotEmpty() && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                if (ActivityCompat.checkSelfPermission(
                        requireContext(),
                        Manifest.permission.ACCESS_FINE_LOCATION
                    ) == PackageManager.PERMISSION_GRANTED
                ) {
                    locationManager.requestLocationUpdates(
                        LocationManager.GPS_PROVIDER,
                        0,
                        0f,
                        locationListener
                    )
                }
            } else {
                Toast.makeText(
                    requireContext(),
                    "Location permission denied. Cannot get current location.",
                    Toast.LENGTH_SHORT
                ).show()
            }
        }
    }
}
