package com.atm.bondowowo.utils

import android.content.Context
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.Job
import kotlinx.coroutines.delay
import kotlinx.coroutines.isActive
import kotlinx.coroutines.launch

class BackgroundJWTChecker {

    private var job: Job? = null

    interface Callback {
        fun onTokenVerified()
        fun onTokenInvalid()
        fun onNetworkError()
    }

    fun startChecking(scope: CoroutineScope, context: Context, callback: Callback) {
        job = scope.launch(Dispatchers.IO) {
            while (isActive) {
                // 2,5 Menit
                delay(2 * 60 * 1000 + 40 * 1000)
                val token = UserPreferencesUtil.getJWT(context)
                if (token != null) {
                    verifyToken(token, callback)
                } else {
                    callback.onTokenInvalid()
                }
            }
        }
    }


    private suspend fun verifyToken(token: String, callback: Callback) {
        try {
            val response = NetworkUtils.apiService.verifyToken("Bearer $token")
            if (response.isSuccessful && response.body() !is String) {
                callback.onTokenVerified()
            } else {
                callback.onTokenInvalid()
            }
        } catch (e: Exception) {
            callback.onNetworkError()
        }
    }

}
