package com.atm.bondowowo.utils

import android.content.Context
import android.widget.Toast
import com.atm.bondowowo.data.local.entity.TransactionEntity
import org.apache.poi.xssf.usermodel.XSSFWorkbook
import java.io.File
import java.io.FileOutputStream

object ExportToFileUtil {

    fun exportDataToFile(
        context: Context,
        transactionEntities: List<TransactionEntity>,
        fileFormat: String
    ) {
        val networkStateLiveData = NetworkStateLiveData(context)
        networkStateLiveData.observeForever { isConnected ->
            if (!isConnected) {
                DialogHelper.showNoInternetDialog(context)
            }
        }

        if (transactionEntities.isNotEmpty()) {
            val workbook = XSSFWorkbook()
            val sheet = workbook.createSheet("Transaction Data")

            val headerRow = sheet.createRow(0)
            val headers = arrayOf("Date", "Name", "Category", "Amount", "Location")
            headers.forEachIndexed { index, headerText ->
                val cell = headerRow.createCell(index)
                cell.setCellValue(headerText)
            }

            transactionEntities.forEachIndexed { rowIndex, transaction ->
                val dataRow = sheet.createRow(rowIndex + 1)
                dataRow.createCell(0).setCellValue(transaction.date.toString())
                dataRow.createCell(1).setCellValue(transaction.nama ?: "")
                dataRow.createCell(2).setCellValue(transaction.kategori ?: "")
                dataRow.createCell(3).setCellValue(transaction.nominal ?: 0.0)
                dataRow.createCell(4).setCellValue(transaction.lokasi ?: "")
            }

            val file = if(fileFormat == "xlsx"){
                File(context.getExternalFilesDir(null), "transaction_data.xlsx")
            } else{
                File(context.getExternalFilesDir(null), "transaction_data.xls")
            }

            val outputStream = FileOutputStream(file)
            workbook.write(outputStream)
            workbook.close()
            outputStream.close()

            Toast.makeText(context, "Data exported to ${file.absolutePath}", Toast.LENGTH_SHORT).show()
//            Toast.makeText(context, file.name, Toast.LENGTH_SHORT).show()

        } else {
            Toast.makeText(context, "No transaction data available", Toast.LENGTH_SHORT).show()
        }
    }
}
