package com.atm.bondowowo.utils

import android.content.Context
import android.content.Intent
import android.widget.Toast
import androidx.core.content.FileProvider
import java.io.File

object SendEmailUtil {

    fun sendEmailWithAttachment(context: Context) {
        val networkStateLiveData = NetworkStateLiveData(context)
        networkStateLiveData.observeForever { isConnected ->
            if (!isConnected) {
                DialogHelper.showNoInternetDialog(context)
            }
        }

        val file = getLatestFileFromStorage(context)

        if (file != null && file.exists()) {
            val uri =
                FileProvider.getUriForFile(context, "${context.packageName}.fileprovider", file)

            val mimeType = if (file.extension == "xlsx") {
                "application/vnd.ms-excel"
            } else {
                "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"

            }

            val emailIntent = Intent(Intent.ACTION_SEND)
            emailIntent.type = mimeType
            emailIntent.putExtra(Intent.EXTRA_EMAIL, arrayOf(UserPreferencesUtil.getEmail(context)))
            emailIntent.putExtra(Intent.EXTRA_SUBJECT, "Export File")
            emailIntent.putExtra(
                Intent.EXTRA_TEXT,
                "Berikut terlampir file transaksi dalam format spreadsheet."
            )
            emailIntent.putExtra(Intent.EXTRA_STREAM, uri)

            emailIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)

            try {
                context.startActivity(Intent.createChooser(emailIntent, "Send mail..."))
            } catch (ex: android.content.ActivityNotFoundException) {
                Toast.makeText(context, "No email clients installed.", Toast.LENGTH_SHORT).show()
            }
        } else {
            Toast.makeText(context, "No transaction file found or attached.", Toast.LENGTH_SHORT)
                .show()
        }
    }

    private fun getLatestFileFromStorage(context: Context): File? {
        val directoryPath = "/storage/emulated/0/Android/data/com.atm.bondowowo/files"
        val directory = File(directoryPath)

        if (!directory.exists() || !directory.isDirectory) {
            return null
        }

        val files =
            directory.listFiles()?.filter { it.isFile }?.sortedByDescending { it.lastModified() }
        return files?.firstOrNull()
    }
}

